#include <ArduinoBLE.h> // Arduino BLE library
#include <Wire.h>
#include <Adafruit_SHT4x.h>

// UUIDs for BLE Service and Characteristics
const char* UUID_serv = "84582cd0-3df0-4e73-9496-29010d7445dd";
const char* UUID_temp1 = "84582cd1-3df0-4e73-9496-29010d7445dd";
const char* UUID_temp2 = "84582cd2-3df0-4e73-9496-29010d7445dd";
const char* UUID_temp3 = "84582cd3-3df0-4e73-9496-29010d7445dd";
const char* UUID_temp4 = "84582cd4-3df0-4e73-9496-29010d7445dd";
const char* UUID_temp5 = "84582cd5-3df0-4e73-9496-29010d7445dd";
const char* UUID_hum1 = "84582cd6-3df0-4e73-9496-29010d7445dd";
const char* UUID_hum2 = "84582cd7-3df0-4e73-9496-29010d7445dd";
const char* UUID_hum3 = "84582cd8-3df0-4e73-9496-29010d7445dd";
const char* UUID_hum4 = "84582cd9-3df0-4e73-9496-29010d7445dd";
const char* UUID_hum5 = "84582cda-3df0-4e73-9496-29010d7445dd";

// BLE Service and Characteristics
BLEService myService(UUID_serv);
BLEFloatCharacteristic chTemp1(UUID_temp1, BLERead | BLENotify);
BLEFloatCharacteristic chTemp2(UUID_temp2, BLERead | BLENotify);
BLEFloatCharacteristic chTemp3(UUID_temp3, BLERead | BLENotify);
BLEFloatCharacteristic chTemp4(UUID_temp4, BLERead | BLENotify);
BLEFloatCharacteristic chTemp5(UUID_temp5, BLERead | BLENotify);
BLEFloatCharacteristic chHum1(UUID_hum1, BLERead | BLENotify);
BLEFloatCharacteristic chHum2(UUID_hum2, BLERead | BLENotify);
BLEFloatCharacteristic chHum3(UUID_hum3, BLERead | BLENotify);
BLEFloatCharacteristic chHum4(UUID_hum4, BLERead | BLENotify);
BLEFloatCharacteristic chHum5(UUID_hum5, BLERead | BLENotify);

Adafruit_SHT4x sht4x[5];
const uint8_t TCA9548A_ADDR = 0x70;

void selectI2CChannel(uint8_t channel) {
    if (channel > 7) return;
    Wire.beginTransmission(TCA9548A_ADDR);
    Wire.write(1 << channel);
    Wire.endTransmission();
}

void setup() {
    Serial.begin(115200);
    uint32_t t = millis();
    while (!Serial) {
        if ((millis() - t) > 5000) break;
    }
    bool err = false;
    pinMode(LED_BUILTIN, OUTPUT);
    digitalWrite(LED_BUILTIN, LOW);

    Wire.begin();

    if (!BLE.begin()) {
        Serial.println("BLE: failed");
        err = true;
    }
    Serial.println("BLE: ok");

    for (uint8_t i = 0; i < 5; i++) {
        selectI2CChannel(i);
        if (!sht4x[i].begin()) {
            Serial.print("SHT4x sensor on channel ");
            Serial.print(i);
            Serial.println(" not found!");
        } else {
            Serial.print("SHT4x sensor on channel ");
            Serial.print(i);
            Serial.println(" initialized successfully.");
        }
    }

    if (err) {
        Serial.println("Init error. System halted");
        while (1) {
            digitalWrite(LED_BUILTIN, HIGH);
            delay(500);
            digitalWrite(LED_BUILTIN, LOW);
            delay(500);
        }
    }

    BLE.setLocalName("SHT4x_Sensor");
    BLE.setAdvertisedService(myService);
    myService.addCharacteristic(chTemp1);
    myService.addCharacteristic(chTemp2);
    myService.addCharacteristic(chTemp3);
    myService.addCharacteristic(chTemp4);
    myService.addCharacteristic(chTemp5);
    myService.addCharacteristic(chHum1);
    myService.addCharacteristic(chHum2);
    myService.addCharacteristic(chHum3);
    myService.addCharacteristic(chHum4);
    myService.addCharacteristic(chHum5);
    BLE.addService(myService);
    BLE.advertise();
    Serial.println("Advertising started");
}

void loop() {
    BLEDevice central = BLE.central();
    if (central) {
        digitalWrite(LED_BUILTIN, HIGH);
        Serial.print("Connected to central: ");
        Serial.println(central.address());
        while (central.connected()) {
            updateValues();
            delay(1000);
        }
        digitalWrite(LED_BUILTIN, LOW);
        Serial.print("Disconnected from central: ");
        Serial.println(central.address());
    }
}

void updateValues() {
    float temperatures[5];
    float humidities[5];
    for (int i = 0; i < 5; i++) {
        selectI2CChannel(i);
        sensors_event_t humidity, temp;
        sht4x[i].getEvent(&humidity, &temp);
        temperatures[i] = temp.temperature;
        humidities[i] = humidity.relative_humidity;
        Serial.print("Temp ");
        Serial.print(i + 1);
        Serial.print(": ");
        Serial.print(temperatures[i]);
        Serial.print(" C, Humidity: ");
        Serial.print(humidities[i]);
        Serial.println(" %");
    }
    chTemp1.writeValue(temperatures[0]);
    chTemp2.writeValue(temperatures[1]);
    chTemp3.writeValue(temperatures[2]);
    chTemp4.writeValue(temperatures[3]);
    chTemp5.writeValue(temperatures[4]);
    chHum1.writeValue(humidities[0]);
    chHum2.writeValue(humidities[1]);
    chHum3.writeValue(humidities[2]);
    chHum4.writeValue(humidities[3]);
    chHum5.writeValue(humidities[4]);
}
